<?php
/*
 * Copyright (c) 2008 Karri "Blaze" Kahelin
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
require_once('utils.php');
require_once('File_Not_Found_Exception.php');

/**
 * Class for globbing directories
 */
class Directory_Scanner {
	protected $db;
	protected $imagedir;
	protected $category;
	protected $directory;
	protected $images;
	protected $subdirs;
	protected $valid = false;
	protected $inDB = NULL;
	protected $inBoth = NULL;

	/**
		Constructs new Directory_Scanner-object containing information
		about provided directory

		@param imagedir path of image root directory
		@param category subdirectory inside root
		@param db 	database access
	 */
	public function __construct($imagedir, $category, Query $db) {
		$this->db = $db;
		$this->imagedir = ensureslash($imagedir);
		$this->category = $category;
		$this->directory = ensureslash($imagedir . $category);
		$this->images = array();
		$this->subdirs = array();
		// since I DON'T want recursive globbing, readdir() wins over
		// glob()
		if(($dir = @opendir($this->directory)) == false) {
			throw new File_Not_Found_Exception($this->directory);
		}
		$this->valid = true;
		while(($file = readdir($dir)) !== false) {
			if($file != '.' && $file != '..') {
				$path = "$this->directory$file";
				if(is_dir($path)) {
					$this->subdirs[] = $file;
				} else if(@getimagesize($path) !== false) {
					$this->images[] = $file;
				}
			}
		}
		closedir($dir);
	}

	/**
	 * Returns name of the directory represented by this object
	 *
	 * @return name of directory
	 */
	public function getDirectory() {
		return $this->directory;
	}

	/**
	 * Returns name of the root directory represented by this object
	 *
	 * @return name of directory
	 */
	public function getImageDirectory() {
		return $this->imagedir;
	}

	/**
	 * Returns name of the subdirectory represented by this object
	 *
	 * @return name of directory
	 */
	public function getCategory() {
		return $this->category;
	}

	/**
	 * Returns array of image filenames contained in this directory
	 *
	 * @return array of image filenames
	 */
	public function getImages() {
		return $this->images();
	}

	/**
	 * Returns array of subdirectory names contained in this directory
	 *
	 * @return array of subdirectory names
	 */
	public function getSubdirs() {
		return $this->subdirs;
	}

	/**
	 * Returns whether this object represents a valid directory
	 *	
	 * @return true if the directory is valid
	 */
	public function isValid() {
		return $this->valid;
	}

	/**
	 * Returns array of image names that are in database, but no longer
	 * on disk
	 *
	 * @return array of deleted image names
	 */
	public function getDeletedImageNames() {
		$this->getImageNamesInDatabase();
		return array_diff($this->inDB, $this->inBoth);
	}

	/**
	 * Returns array of image names that are on disk, but not yet in
	 * database
	 *
	 * @return array of new image names
	 */
	public function getNewImageNames() {
		$this->getImageNamesInDatabase();
		return array_diff($this->images, $this->inBoth);
	}

	/**
	 * Fetches image names from database
	 */
	protected function getImageNamesInDatabase() {
		if($this->inDB === NULL) {
			$this->inDB = $this->db->getAllImageNamesInCategory(
				$this->category);
			$this->inBoth = array_intersect($this->inDB,
				$this->images);
		}
	}
}
?>
