<?php
/*
 * Copyright (c) 2008 Karri "Blaze" Kahelin
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
require_once('Not_Connected_Exception.php');
require_once('Query.php');

/**
 * Database frontend class for PostgreSQL
 *
 * All methods override superclass ones, see superclass for documentation.
 */
class PostgresQuery extends Query {
	protected function __construct($prefix) {
		parent::__construct($prefix);
		$colnames = array('filename', 'category', 'added', 'taken',
			'width', 'height', 'cam_manufacturer', 'cam_model',
			'shutter_speed', 'subject_distance', 'focal_length',
			'exposure_time', 'location', 'photographer',
			'description', 'format', 'size');

		$this->s['create_tables'] =
			"CREATE TABLE %prefix%pictures (
			id SERIAL NOT NULL ,
			filename TEXT NOT NULL ,
			category TEXT NOT NULL ,
			added TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
			taken TIMESTAMP NULL DEFAULT NULL ,
			width SMALLINT NULL DEFAULT NULL ,
			height SMALLINT NULL DEFAULT NULL ,
			cam_manufacturer VARCHAR( 255 ) NULL DEFAULT NULL ,
			cam_model VARCHAR( 255 ) NULL DEFAULT NULL ,
			shutter_speed VARCHAR( 255 ) NULL DEFAULT NULL ,
			subject_distance VARCHAR( 255 ) NULL DEFAULT NULL ,
			focal_length VARCHAR( 255 ) NULL DEFAULT NULL ,
			exposure_time VARCHAR( 255 ) NULL DEFAULT NULL ,
			location VARCHAR( 255 ) NULL DEFAULT NULL ,
			photographer VARCHAR( 255 ) NULL DEFAULT NULL ,
			description TEXT DEFAULT NULL ,
			format VARCHAR( 64 ) NOT NULL ,
			size INT NOT NULL ,
			PRIMARY KEY ( id ) 
			);

			CREATE TABLE %prefix%comments (
			id SERIAL NOT NULL ,
			pic_id INT NOT NULL ,
			comment TEXT NOT NULL ,
			username TEXT NOT NULL ,
			added TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
			FOREIGN KEY ( pic_id )
			REFERENCES %prefix%pictures ( id )
			ON DELETE CASCADE
			);

			CREATE TABLE %prefix%tags (
			id SERIAL NOT NULL ,
			name VARCHAR( 255 ) NOT NULL ,
			PRIMARY KEY ( id )
			);

			CREATE TABLE %prefix%tag_pic (
			tag_id INT NOT NULL ,
			pic_id INT NOT NULL ,
			FOREIGN KEY ( tag_id )
			REFERENCES %prefix%tags ( id )
			ON DELETE CASCADE ,
			FOREIGN KEY ( pic_id )
			REFERENCES %prefix%pictures ( id )
			ON DELETE CASCADE
			);
			
			CREATE INDEX comment_index ON %prefix%comments (pic_id);
			CREATE INDEX tag_id_index ON %prefix%tag_pic (tag_id);
			CREATE INDEX pic_id_index ON %prefix%tag_pic (pic_id);
			";
		$this->s['get_image_names'] = "SELECT filename FROM %prefix%" .
			"pictures WHERE category=:cat LIMIT :length " .
		        "OFFSET :start;";
		$this->s['get_images_by_name'] = "SELECT " . implode(', ',
			$colnames) . " FROM %prefix%pictures WHERE " .
			"category=:cat LIMIT :length OFFSET :start;";
		$this->s['get_image_names_by_criteria'] = "SELECT DISTINCT " .
			"%prefix%pictures.category, %prefix%pictures.filename" .
			" FROM %prefix%pictures, " .
			"%prefix%tag_pic, %prefix%tags WHERE " .
			"(%prefix%tags.name LIKE :search AND " .
			"%prefix%tag_pic.pic_id = %prefix%pictures.id AND " .
			"%prefix%tag_pic.tag_id = %prefix%tags.id) OR " .
			"(%prefix%pictures.description LIKE :search) OR " .
			"(%prefix%pictures.filename LIKE :search) OR " .
			"(%prefix%pictures.category LIKE :search) OR " .
			"(%prefix%pictures.location LIKE :search) OR " .
			"(%prefix%pictures.photographer LIKE :search) OR " .
			"(%prefix%pictures.description LIKE :search) " .
			"GROUP BY %prefix%pictures.id LIMIT :length " .
		        "OFFSET :start;";
	}

	public function createTables() {
		if(!$this->tablesExist()) {
			// *yawn* postgre has its own share of lameness, too
			// it can only create one table per query, so we need
			// to split:
			$queries = explode(');', $this->s['create_tables']);
			foreach($queries as $q) {
				if(strlen(trim($q)) == 0) {
					continue;
				}
				$stmt = $this->pdo->query(trim($q) . ');');
				if($stmt === false) {
					throw new PDOException(implode(' ',
						$this->pdo->errorInfo()));
				}
			}
		}
	}

	public function addImage($category, $name, array $attrs) {
		// Postgre doesn't recognize our dates, so we need to format
		// them specially for it
		if(isset($attrs['taken'])) {
			$attrs['taken'] = date('r', strtotime($attrs['taken']));
		}
		parent::addImage($category, $name, $attrs);
	}
}
