<?php
/*
 * Copyright (c) 2008-2009 Karri "Blaze" Kahelin
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
require_once('Not_Connected_Exception.php');
require_once('MysqlQuery.php');
require_once('PostgresQuery.php');
require_once('SqliteQuery.php');

/**
 * Database frontend class for generic database
 */
class Query {
	protected $connected = false;
	protected $pdo = NULL;
	protected $s = array();

	/**
	 * Returns new instance of Query, or one of its subclasses. This method
	 * may also be given the same parameters as the connect()-method, and it
	 * will call connect() with those parameters.
	 * Such an arrangement is necessary because database syntax and support
	 * just varies too much.
	 *
	 * @param prefix prefix to apply to tables
	 */
	static public function create($prefix) {
		$instance = NULL;
		// determine, which type of Query to create
		$args = func_get_args();
		if(count($args) == 4) {
			if(strpos($args[1], 'mysql') !== false) {
				$instance = new MysqlQuery($prefix, $args[1],
					$args[2], $args[3]);
			} else if(strpos($args[1], 'pgsql') !== false) {
				$instance = new PostgresQuery($prefix, $args[1],
					$args[2], $args[3]);
			} else if(strpos($args[1], 'sqlite') !== false) {
				$instance = new SqliteQuery($prefix, $args[1],
					$args[2], $args[3]);
			} else {
				$instance = new Query($prefix, $args[1],
					$args[2], $args[3]);
			}
		} else {
			$instance = new Query($prefix);
		}
		if($instance != NULL) {
			$instance->initialize($args);
		}
		return $instance;
	}

	/**
	 * Constructs new Query-object. Constructor may also be given the same
	 * parameters as the connect()-method, and it will call	connect() with
	 * those parameters.
	 *
	 * @param prefix prefix to apply to tables
	 */
	protected function __construct($prefix) {
		$colnames = array('filename', 'category', 'added', 'taken',
			'width', 'height', 'cam_manufacturer', 'cam_model',
			'shutter_speed', 'subject_distance', 'focal_length',
			'exposure_time', 'location', 'photographer',
			'description', 'format', 'size');
		$this->s['create_tables'] =
			"CREATE TABLE %prefix%pictures (
			id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL ,
			filename TEXT NOT NULL ,
			category TEXT NOT NULL ,
			added TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
			taken TIMESTAMP NULL DEFAULT NULL ,
			width SMALLINT UNSIGNED NULL DEFAULT NULL ,
			height SMALLINT UNSIGNED NULL DEFAULT NULL ,
			cam_manufacturer VARCHAR( 255 ) NULL DEFAULT NULL ,
			cam_model VARCHAR( 255 ) NULL DEFAULT NULL ,
			shutter_speed VARCHAR( 255 ) NULL DEFAULT NULL ,
			subject_distance VARCHAR( 255 ) NULL DEFAULT NULL ,
			focal_length VARCHAR( 255 ) NULL DEFAULT NULL ,
			exposure_time VARCHAR( 255 ) NULL DEFAULT NULL ,
			location VARCHAR( 255 ) NULL DEFAULT NULL ,
			photographer VARCHAR( 255 ) NULL DEFAULT NULL ,
			description TEXT NULL DEFAULT NULL ,
			format VARCHAR( 64 ) NOT NULL ,
			size INT UNSIGNED NOT NULL
			);

			CREATE TABLE %prefix%comments (
			id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL ,
			pic_id INT UNSIGNED NOT NULL ,
			comment TEXT NOT NULL ,
			username TEXT NOT NULL ,
			added TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
			FOREIGN KEY ( pic_id )
			REFERENCES %prefix%pictures ( id )
			ON DELETE CASCADE
			);

			CREATE TABLE %prefix%tags (
			id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL ,
			name VARCHAR( 255 ) NOT NULL
			);

			CREATE TABLE %prefix%tag_pic (
			tag_id INT UNSIGNED NOT NULL ,
			pic_id INT UNSIGNED NOT NULL ,
			FOREIGN KEY ( tag_id )
			REFERENCES %prefix%tags ( id )
			ON DELETE CASCADE ,
			FOREIGN KEY ( pic_id )
			REFERENCES %prefix%pictures ( id )
			ON DELETE CASCADE
			);";
		$this->s['add_image'] = "INSERT INTO %prefix%pictures " .
			"(%columns%) VALUES (%values%);";
		$this->s['change_image'] = "UPDATE %prefix%pictures SET " .
			"%values% WHERE id=:id;";
		$this->s['delete_images_by_id'] = "DELETE FROM " .
			"%prefix%pictures WHERE id IN (:ids);";
		$this->s['delete_images_by_name'] = "DELETE FROM " .
			"%prefix%pictures WHERE category=:cat AND filename " .
			"IN (:names);";
		$this->s['get_categories'] = "SELECT DISTINCT category FROM " .
			"%prefix%pictures;";
		$this->s['get_image_count'] = "SELECT COUNT(*) FROM " .
			"%prefix%pictures;";
		$this->s['get_image_count_in_category'] = "SELECT COUNT(*) " .
			"FROM %prefix%pictures WHERE category=:cat;";
		$this->s['get_all_image_names'] = "SELECT filename FROM " .
			"%prefix%pictures WHERE category=:cat;";
		$this->s['get_image_names'] = "SELECT filename FROM %prefix%" .
			"pictures WHERE category=:cat LIMIT :start, :length;";
		$this->s['get_images_by_name'] = "SELECT " . implode(', ',
			$colnames) . " FROM %prefix%pictures WHERE " .
			"category=:cat LIMIT :start, :length;";
		$this->s['get_image_by_name'] = "SELECT " . implode(', ',
			$colnames) . " FROM %prefix%pictures WHERE " .
			"category=:cat AND filename=:name LIMIT 1;";
		$this->s['get_all_image_names_by_criteria'] = "SELECT DISTINCT ".
			"%prefix%pictures.category, %prefix%pictures.filename" .
			" FROM %prefix%pictures, " .
			"%prefix%tag_pic, %prefix%tags WHERE " .
			"(%prefix%tags.name LIKE :search AND " .
			"%prefix%tag_pic.pic_id = %prefix%pictures.id AND " .
			"%prefix%tag_pic.tag_id = %prefix%tags.id) OR " .
			"(%prefix%pictures.description LIKE :search) OR " .
			"(%prefix%pictures.filename LIKE :search) OR " .
			"(%prefix%pictures.category LIKE :search) OR " .
			"(%prefix%pictures.location LIKE :search) OR " .
			"(%prefix%pictures.photographer LIKE :search) OR " .
			"(%prefix%pictures.description LIKE :search) " .
			"GROUP BY %prefix%pictures.id";
		$this->s['get_image_names_by_criteria'] = "SELECT DISTINCT " .
			"%prefix%pictures.category, %prefix%pictures.filename" .
			" FROM %prefix%pictures, " .
			"%prefix%tag_pic, %prefix%tags WHERE " .
			"(%prefix%tags.name LIKE :search AND " .
			"%prefix%tag_pic.pic_id = %prefix%pictures.id AND " .
			"%prefix%tag_pic.tag_id = %prefix%tags.id) OR " .
			"(%prefix%pictures.description LIKE :search) OR " .
			"(%prefix%pictures.filename LIKE :search) OR " .
			"(%prefix%pictures.category LIKE :search) OR " .
			"(%prefix%pictures.location LIKE :search) OR " .
			"(%prefix%pictures.photographer LIKE :search) OR " .
			"(%prefix%pictures.description LIKE :search) " .
			"GROUP BY %prefix%pictures.id LIMIT :start, :length;";
		$this->s['get_image_count_by_criteria'] = "SELECT DISTINCT ".
			"COUNT(*)" .
			" FROM %prefix%pictures, " .
			"%prefix%tag_pic, %prefix%tags WHERE " .
			"(%prefix%tags.name LIKE :search AND " .
			"%prefix%tag_pic.pic_id = %prefix%pictures.id AND " .
			"%prefix%tag_pic.tag_id = %prefix%tags.id) OR " .
			"(%prefix%pictures.description LIKE :search) OR " .
			"(%prefix%pictures.filename LIKE :search) OR " .
			"(%prefix%pictures.category LIKE :search) OR " .
			"(%prefix%pictures.location LIKE :search) OR " .
			"(%prefix%pictures.photographer LIKE :search) OR " .
			"(%prefix%pictures.description LIKE :search) " .
			"GROUP BY %prefix%pictures.id";
		$this->s['get_image_id_by_name'] = "SELECT id FROM " .
			"%prefix%pictures WHERE category=:cat AND " .
			"filename=:name LIMIT 1;";
		$this->s['insert_comment'] = "INSERT INTO %prefix%comments" .
			"(pic_id, comment, username) VALUES (:pic_id, " .
			":comment, :user);";
		$this->s['delete_comment'] = "DELETE FROM %prefix%comments " .
			"WHERE id=:id;";
		$this->s['change_comment'] = 'UPDATE %prefix%comments SET ' .
			"username=:user, comment=:comment WHERE id=:id;";
		$this->s['get_comments'] = "SELECT id, username, comment, " .
			"added FROM %prefix%comments WHERE pic_id=:pic_id " .
			"ORDER BY added ASC;";
		$this->s['get_tag_names_by_image_id'] = "SELECT %prefix%tags." .
			"name FROM %prefix%tags, %prefix%tag_pic WHERE " .
			"%prefix%tag_pic.pic_id=:id AND %prefix%tags.id=" .
			"%prefix%tag_pic.tag_id GROUP BY %prefix%" .
			"tags.name;";
		$this->s['get_tag_id_by_name'] = "SELECT id FROM " .
			"%prefix%tags WHERE name=:name LIMIT 1;";
		$this->s['delete_tags_by_image_id'] = "DELETE FROM " .
			"%prefix%tag_pic WHERE pic_id=:id;";
		$this->s['add_tag_for_image'] = "INSERT INTO " .
			"%prefix%tag_pic (pic_id, tag_id) VALUES(:pic_id, " .
			":tag_id);";
		$this->s['create_tag'] = "INSERT INTO %prefix%tags (name) " .
			"VALUES(:name);";
		$this->s['remove_tags_by_id'] = 'DELETE FROM %prefix%tags ' .
			'WHERE id IN (:ids);';
		$this->s['remove_tags_by_name'] = 'DELETE FROM %prefix%tags ' .
			'WHERE name IN (:names);';
		$this->s['get_unused_tag_ids'] = 'SELECT id FROM %prefix%tags' .
			' WHERE id NOT IN (SELECT DISTINCT tag_id FROM ' .
			'%prefix%tag_pic);';
	}

	/**
	 * Initializes the Query-object
	 *
	 * @param args array of arguments given to constructor
	 */
	protected function initialize(array $args) {
		foreach($this->s as $key => $val) {
			$this->s[$key] = str_replace('%prefix%', $args[0],
				$val);
		}

		if(count($args) == 4) {
			if($this->connect($args[1], $args[2], $args[3])) {
				$this->createTables();
			}
		}
	}

	/**
	 * Destructs the Query-object. Disconnects from database, if the
	 * object was connected.
	 */
	public function __destruct() {
		$this->disconnect();
	}

	/**
	 * Connects to a database. This method is always safe to call, in case
	 * the object is already connected, it will disconnect and then
	 * reconnect with new values.
	 *
	 * @param dsn 	data source connection string
	 * @param user 	username for database
	 * @param pass 	password for database
	 *
	 * @return true if connecting was successful
	 */
	public function connect($dsn, $user, $pass) {
		if($this->connected) {
			$this->disconnect();
		}

		try {
			$this->pdo = new PDO($dsn, $user, $pass);
		} catch(PDOException $e) {
			throw new PDOException($e->getMessage());
		}
			
		return ($this->connected = true);
	}

	/**
	 * Disconnects from database. This method is always safe to call, in
	 * case the object is not connected the method does nothing.
	 */
	public function disconnect() {
		if($this->connected) {
			unset($this->pdo);
			$this->connected = false;
		}
	}

	/**
	 * Returns whether this Query-object is connected to the database
	 *
	 * @return true if connected to a database
	 */
	public function isConnected() {
		return $this->connected;
	}

	/**
	 * Returns whether tables already exist in the database
	 * 
	 * @return true if tables exist
	 */
	public function tablesExist() {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		// lame way to check, but at least SQLite doesn't support
		// anything better
		if($this->pdo->query($this->s['get_image_count']) === false) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Creates tables to database. This method is always safe to call, in
	 * case the tables already exist, nothing is done.
	 */
	public function createTables() {
		if(!$this->tablesExist()) {
			$stmt = $this->pdo->query($this->s['create_tables']);
			if($stmt === false) {
				throw new PDOException(implode(' ',
					$this->pdo->errorInfo()));
			}
		}
	}

	/**
	 * Adds a single image to database
	 *
	 * @param category 	category (path) of the image
	 * @param name 		filename of the image
	 * @param attrs 	associative array of other initial image
	 * 			information
	 */
	public function addImage($category, $name, array $attrs) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$attrs['filename'] = $name;
		$attrs['category'] = $category;
		foreach($attrs as $key => $val) {
			$attrs[$key] = $this->pdo->quote($val);
		}

		// construct query
		$sql = $this->s['add_image'];
		$sql = str_replace('%columns%', implode(', ',
			array_keys($attrs)), $sql);
		$sql = str_replace('%values%', implode(', ', $attrs), $sql);

		if($this->pdo->query($sql) === false) {
			throw new PDOException(implode(' ',
				$this->pdo->errorInfo()));
		}
	}

	/**
	 * Changes images information in the database
	 *
	 * @param id    id of the image
	 * @param attrs new set of attributes to save for the image
	 */
	public function changeImage($id, array $attrs) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$values = array();
		foreach($attrs as $key => $val) {
			if($val === '') {
				$values[] = $key . ' = NULL';
			} else {
				$values[] = $key . ' = ' .
					$this->pdo->quote($val);
			}
		}
		$sql = $this->s['change_image'];
		$sql = str_replace('%values%', implode(', ', $values), $sql);
		$stmt = $this->pdo->prepare($sql);
		$stmt->bindValue(':id', $id);
		if($stmt->execute() === false) {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Deletes images from database based on their IDs
	 *
	 * @param images array of image IDs
	 */
	public function deleteImagesById(array $images) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$ids = implode(", ", $images);
		$sql = str_replace(':ids', $ids,
			$this->s['delete_images_by_id']);
		$stmt = $this->pdo->prepare($sql);
		if($stmt->execute() === false) {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Deletes images from database based on their name
	 *
	 * @param category 	category of images
	 * @param images 	array of image names
	 */
	public function deleteImagesByName($category, array $images) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$names = "'" . implode("', '", $images) . "'";
		$sql = str_replace(':names', $names,
			$this->s['delete_images_by_name']);
		$stmt = $this->pdo->prepare($sql);
		$stmt->bindValue(':cat', $category);
		if($stmt->execute() === false) {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns list of categories
	 *
	 * @return array of categories
	 */
	public function getCategories() {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		if(($stmt = $this->pdo->query($this->s['get_categories'])) !==
				false) {
			return $stmt->fetchAll(PDO::FETCH_COLUMN, 0);
		} else {
			throw new PDOException(implode(' ',
				$this->pdo->errorInfo()));
		}
	}

	/**
	 * Returns the amount of images in database
	 *
	 * @return amount of images in database
	 */
	public function getImageCount() {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		if(($stmt = $this->pdo->query($this->s['get_image_count'])) !==
				false) {
			$return = $stmt->fetch(PDO::FETCH_COLUMN, 0);
			$stmt->closeCursor();
			return $return;
		} else {
			throw new PDOException(implode(' ',
				$this->pdo->errorInfo()));
		}
	}

	/**
	 * Returns list of image file names under a category (non-recursive)
	 *
	 * @param category category of images
	 *
	 * @return array of image file names
	 */
	public function getAllImageNamesInCategory($category) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['get_all_image_names']);
		$stmt->bindValue(':cat', $category);
		if($stmt->execute() === true) {
			return $stmt->fetchAll(PDO::FETCH_COLUMN, 0);
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns portion of list of image file names under a category
	 * (non-recursive)
	 * 
	 * @param category 	category of images
	 * @param start 	offset in list
	 * @param length 	amount of names to return
	 *
	 * @return array of arrays containing image file names and categories
	 */
	public function getImageNamesInCategory($category, $start, $length) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['get_image_names']);
		$stmt->bindValue(':cat', $category);
		$stmt->bindValue(':start', (int) $start, PDO::PARAM_INT);
		$stmt->bindValue(':length', (int) $length, PDO::PARAM_INT);
		if($stmt->execute() === true) {
			$return = $stmt->fetchAll(PDO::FETCH_COLUMN, 0);
			for($i = 0; $i < count($return); ++$i) {
				$return[$i] = array($category, $return[$i]);
			}
			return $return;
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns portion of list of images under a category (non-recursive)
	 *
	 * @param category 	category of images
	 * @param start 	offset in list
	 * @param length 	amount of images to return
	 *
	 * @return array of images information
	 */
	public function getImagesInCategory($category, $start, $length) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['get_images_by_name']);
		$stmt->bindValue(':cat', $category);
		$stmt->bindValue(':start', (int) $start, PDO::PARAM_INT);
		$stmt->bindValue(':length', (int) $length, PDO::PARAM_INT);
		if($stmt->execute() === true) {
			return $stmt->fetchAll(PDO::FETCH_ASSOC);
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns count of images in category (non-recursive)
	 *
	 * @param category category of images
	 *
	 * @return count of images
	 */
	public function getImageCountInCategory($category) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s[
			'get_image_count_in_category']);
		$stmt->bindValue(':cat', $category);
		if($stmt->execute() === true) {
			$return = (int)$stmt->fetchColumn();
			$stmt->closeCursor();
			return $return;
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns information of single image
	 *
	 * @param category 	category of image
	 * @param name 		filename of image
	 *
	 * @return information about the image (associative array)
	 */
	public function getImageByName($category, $name) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['get_image_by_name']);
		$stmt->bindValue(':cat', $category);
		$stmt->bindValue(':name', $name);
		if($stmt->execute() === true) {
			$return = $stmt->fetch(PDO::FETCH_ASSOC);
			$stmt->closeCursor();
			return $return;
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns ID per given category and name
	 *
	 * @param category 	category of image
	 * @param name 		filename of image
	 *
	 * @return ID of image, or 0 if no such image was found
	 */
	public function getImageIdByName($category, $name) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['get_image_id_by_name']);
		$stmt->bindValue(':cat', $category);
		$stmt->bindValue(':name', $name);
		if($stmt->execute() === true) {
			$return = (int)$stmt->fetchColumn();
			$stmt->closeCursor();
			return $return;
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns image names matching given search criteria
	 *
	 * @param search criteria
	 *
	 * @return array of arrays containing categories and image names
	 */
	public function getAllImageNamesByCriteria($search) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s[
			'get_all_image_names_by_criteria']);
		$stmt->bindValue(':search', "%$search%");
		if($stmt->execute() === true) {
			return $stmt->fetchAll(PDO::FETCH_NUM);
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}

	}

	/**
	 * Returns image names matching given search criteria
	 *
	 * @param search criteria
	 * @param start  offset in list
	 * @param length amount of names to return
	 *
	 * @return array of arrays containing categories and image names
	 */
	public function getImageNamesByCriteria($search, $start, $length) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s[
			'get_image_names_by_criteria']);
		$stmt->bindValue(':search', "%$search%");
		$stmt->bindValue(':start', (int)$start, PDO::PARAM_INT);
		$stmt->bindValue(':length', (int)$length, PDO::PARAM_INT);
		if($stmt->execute() === true) {
			return $stmt->fetchAll(PDO::FETCH_NUM);
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns count of images matching given search criteria
	 *
	 * @param search criteria
	 *
	 * @return count of images
	 */
	public function getImageCountByCriteria($search) {
		/*
		 * For whatever reason the proper query doesn't work (always returs 1)
		 * and I'm far too lazy to find out why, so I'll take the cheap way out
		 */
		return count($this->getAllImageNamesByCriteria($search));
		/*
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s[
			'get_image_count_by_criteria']);
		$stmt->bindValue(':search', "%$search%");
		if($stmt->execute() === true) {
			//$return = (int)$stmt->fetchColumn();
			$return = $stmt->fetch(PDO::FETCH_COLUMN, 0);
			$stmt->closeCursor();
			var_dump($return);
			return $return;
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
		 */
	}

	/**
	 * Inserts a comment
	 *
	 * @param user_com 	name of comment author
	 * @param comment_com 	comment text
	 * @param pic_id 	id of image comment relates to
	 */
	public function insertComment($user_com, $comment_com, $pic_id) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['insert_comment']);
		$stmt->bindValue(':pic_id', $pic_id);
		$stmt->bindValue(':comment', $comment_com);
		$stmt->bindValue(':user', $user_com);
		if($stmt->execute() !== true) {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Deletes a comment
	 *
	 * @param id id of the comment
	 */
	public function deleteComment($id) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['delete_comment']);
		$stmt->bindValue(':id', $id);
		if($stmt->execute() !== true) {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Changes comment in the database
	 *
	 * @param id      id of the comment
	 * @param user    new user name
	 * @param comment new comment text
	 */
	public function changeComment($id, $user, $comment) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['change_comment']);
		$stmt->bindValue(':id', $id);
		$stmt->bindValue(':user', $user);
		$stmt->bindValue(':comment', $comment);
		if($stmt->execute() !== true) {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Gets comments belonging to specified picture
	 *
	 * @param pic_id id of the image
	 *
	 * @return array of comments (array of zero items, if the specified
	 *         image has no comments, or the image does not exist)
	 */
	public function getComments($pic_id) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['get_comments']);
		$stmt->bindValue(':pic_id', $pic_id);
		if($stmt->execute() === true) {
			return $stmt->fetchAll(PDO::FETCH_ASSOC);
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Gets all tag names belonging to specified picture
	 *
	 * @param id id of the image
	 *
	 * @return array of tag names (array of zero items, if the specified
	 *         image has no tags, or the image does not exist
	 */
	public function getTagNamesByImageId($id) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s[
			'get_tag_names_by_image_id']);
		$stmt->bindValue(':id', $id);
		if($stmt->execute() === true) {
			return $stmt->fetchAll(PDO::FETCH_COLUMN, 0);
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns ID of a tag
	 *
	 * @param name name of tag
	 *
	 * @return id of tag, or 0 if no such tag exists
	 */
	public function getTagIdByName($name) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['get_tag_id_by_name']);
		$stmt->bindValue(':name', $name);
		if($stmt->execute() === true) {
			$return = (int)$stmt->fetch(PDO::FETCH_COLUMN, 0);
			$stmt->closeCursor();
			return $return;
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Changes tags an image is associated with
	 *
	 * @param name id   	id of image
	 * @param name tags 	array of new tag names to associate to image
	 */
	public function changeTags($id, $tags) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		// diffing the set of old and new tags is too difficult so we
		// just delete all old associations
		$stmt = $this->pdo->prepare($this->s[
			'delete_tags_by_image_id']);
		$stmt->bindValue(':id', $id);
		if($stmt->execute() === false) {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
		// and then insert new ones
		$stmt = $this->pdo->prepare($this->s['add_tag_for_image']);
		foreach($tags as $tag) {
			if(strlen(trim($tag)) == 0) {
				continue;
			}
			$tid = $this->getTagIdByName($tag);
			if($tid == 0) {
				// tag doesn't exist, create it
				$cst = $this->pdo->prepare(
					$this->s['create_tag']);
				$cst->bindValue(':name', $tag);
				if($cst->execute() === true) {
					$cst->fetch(PDO::FETCH_COLUMN, 0);
					$tid = $this->pdo->lastInsertId();
					$cst->closeCursor();
				} else {
					throw new PDOException(implode(' ',
						$cst->errorInfo()));
				}
			}
			$stmt->bindValue(':pic_id', $id);
			$stmt->bindValue(':tag_id', $tid);
			if($stmt->execute() === false) {
				throw new PDOException(implode(' ',
					$stmt->errorInfo()));
			}
		}
	}

	/**
	 * Removes tags from database by ID
	 *
	 * @param tags array of tag IDs
	 */
	public function removeTagsById(array $tags) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		if(count($tags) > 0) {
			$stmt = $this->pdo->prepare(str_replace(':ids',
				implode(', ', $tags),
				$this->s['remove_tags_by_id']));
			if($stmt->execute() === false) {
				throw new PDOException(implode(' ',
					$stmt->errorInfo()));
			}
		}
	}

	/**
	 * Removes tags from database by name
	 *
	 * @param tags array of tag names
	 */
	public function removeTagsByName(array $tags) {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		if(count($tags) > 0) {
			$stmt = $this->pdo->prepare(str_replace(':names',
				implode(', ', $tags),
				$this->s['remove_tags_by_name']));
			if($stmt->execute() === false) {
				throw new PDOException(implode(' ',
					$stmt->errorInfo()));
			}
		}
	}

	/**
	 * Gets IDs of unused tags in database
	 *
	 * @return array containing IDs of unused tags
	 */
	public function getUnusedTagIds() {
		if(!$this->isConnected()) {
			throw new Not_Connected_Exception();
		}
		$stmt = $this->pdo->prepare($this->s['get_unused_tag_ids']);
		if($stmt->execute() === true) {
			$return = $stmt->fetchAll(PDO::FETCH_COLUMN, 0);
			for($i = 0; $i < count($return); $return[$i++] =
				(int)$return[$i]);
			return $return;
		} else {
			throw new PDOException(implode(' ',
				$stmt->errorInfo()));
		}
	}

	/**
	 * Returns the driver name this instance of Query is using
	 *
	 * @return database driver name
	 */
	public function getDriverName() {
		return $this->pdo->getAttribute(PDO::ATTR_DRIVER_NAME);
	}

	/**
	 * Returns the driver version this instance of Query is using
	 *
	 * @return database driver version
	 */
	public function getDriverVersion() {
		return $this->pdo->getAttribute(PDO::ATTR_CLIENT_VERSION);
	}
}
?>
