<?php
/*
 * Copyright (c) 2008-2009 Karri "Blaze" Kahelin
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
@include_once('imagecreatefrombmp.php');
require_once('utils.php');
require_once('Settings.php');
require_once('Query.php');
require_once('Strings.php');
require_once('Submit_Handler.php');
require_once('Directory_Scanner.php');
require_once('View.php');
require_once('File_Not_Found_Exception.php');
require_once('Image_Exception.php');

/**
 * Main SolGal class
 */
class Sol_Gal {
	protected $s = NULL;
	protected $conf = NULL;
	protected $query = NULL;
	protected $errors = array();
	protected $notices = array();
	private $_loggedIn = false;

	/**
	 * Constructs new Sol_Gal
	 *
	 * @param config_file 	configuration file to use
	 * @param index 	whether to perform indexing
	 * @param display 	whether to display page
	 */
	public function __construct($config_file = 'LocalSettings.php',
			$index = true, $display = true) {
		// get resources
		$this->s = new Strings();
		// get settings
		try {
			$this->conf = new Settings($config_file);
		} catch(File_Not_Found_Exception $e) {
			$this->errors[] = $this->s->get
				('config_file_not_found_error') .
				$e->getMessage();
			$this->conf = new Settings(NULL);
		}
		// check if we need to start a session
		if(isset($_GET[$this->conf->get('prefix') . 'session']) &&
				$_GET[$this->conf->get('prefix') .
				'session'] == 'yes' && session_id() === '') {
			session_start();
				}
		// create DAO object
		try {
			$this->query = Query::create($this->conf->get('prefix'),
				$this->conf->get('dbdsn'),
				$this->conf->get('dbuser'),
				$this->conf->get('dbpass'));
		} catch(PDOException $e) {
			$this->errors[] = $this->s->get('db_connect_error') . 
				$e->getMessage();
		}
		if($this->query !== NULL) {
			// handle form submits
			try {
				$sh = new Submit_Handler($this, $this->s,
					$this->query,
					$this->conf->get('prefix'));
				$sh->handleSubmits();
			} catch(Comment_Exception $e) {
				$this->errors[] = $this->s->get('comment_error')
					. $e->getMessage();
			} catch(Admin_Exception $e) {
				$this->errors[] = $this->s->get('admin_error') .
					$e->getMessage();
			}
			if($sh->getGivenPass() !== false) {
				// check if the right password was given
				if($this->conf->get('admin_pass') ==
						$sh->getGivenPass()) {
					$this->_loggedIn = true;
				} else {
					$this->errors[] =
						$this->s->get('wrong_password');
				}
			}
		}
		// index
		if($index) {
			$this->indexAll();
			$this->removeUnusedTags();
		}
		// construct page
		if($display) {
			$this->doPage();
		}
	}

	/**
	 * Destructs a Sol_Gal-object
	 */
	public function __destruct() {
		unset($this->query);
		unset($this->conf);
		unset($this->s);
	}

	/**
	 * Displays a error message on generated page
	 *
	 * @param message error message
	 */
	public function displayError($message) {
		$this->errors[] = $message;
	}

	/**
	 * Displays an informational message on generated page
	 *
	 * @param message message
	 */
	public function displayNotice($message) {
		$this->notices[] = $message;
	}

	/**
	 * Indexes all images under configured image dir
	 */
	public function indexAll() {
		if($this->query === NULL) {
			return false;
		}

		ignore_user_abort(true);
		set_time_limit(0);
		try {
			// create root dir node and index that recursively
			$this->indexDirectory(new Directory_Scanner(
				$this->conf->get('image_dir'), '',
				$this->query));
		} catch(File_Not_Found_Exception $e) {
			$this->errors[] = $this->s->get(
				'invalid_directory_error') . $e->getMessage();
			return false;
		}
		ignore_user_abort(false);
	}

	/**
	 * Removes unused tags
	 */
	public function removeUnusedTags() {
		if($this->query === NULL) {
			return false;
		}
		try {
			$ids = $this->query->getUnusedTagIds();
			if(count($ids) > 0) {
				$this->query->removeTagsById($ids);
			}
		} catch(Exception $e) {
			$this->errors[] = $this->s->get('internal_error') .
				$e->getMessage();
			return false;
		}
	}

	/**
	 * Indexes a directory and it's subdirectories
	 *
	 * @param dir Directory_Scanner of root directory
	 */
	protected function indexDirectory(Directory_Scanner $dir) {
		try {
			$deleted = $dir->getDeletedImageNames();
			$new = $dir->getNewImageNames();
		} catch(Exception $e) {
			$this->errors[] = $this->s->get('index_error') .
				$e->getMessage();
			return false;
		}

		// remove deleted from DB
		try {
			$this->query->deleteImagesByName($dir->getCategory(),
				$deleted);
		} catch(Exception $e) {
			$this->errors[] = $this->s->get('index_error') .
				$e->getMessage();
		}
		// remove thumbs from disk
		foreach($deleted as $img) {
			@unlink($this->conf->get('thumb_dir') . $img .
				'_sm.jpg');
			@unlink($this->conf->get('thumb_dir') . $img .
				'_nm.jpg');
			// TODO: remove dir, if empty
		}

		// add new ones
		foreach($new as $img) {
			try {
				$this->index($dir->getCategory(), $img);
			} catch(Exception $e) {
				$this->errors[] = $this->s->get('index_error') .
					$e->getMessage();
			}
		}

		$subdirs = $dir->getSubdirs();
		foreach($subdirs as $subdir) {
			try {
				$scanner = new Directory_Scanner(
					$this->conf->get('image_dir'),
					ensureslash($dir->getCategory()) .
					$subdir, $this->query);
			} catch(File_Not_Found_Exception $e) {
				// should actually never happen
				$this->errors[] = $this->s->get(
					'invalid_directory_error') .
					$e->getMessage();
				return false;
			}
			$this->indexDirectory($scanner);
		}
	}

	/**
	 * Indexes a single image
	 *
	 * @param category 	category of the image
	 * @param name 		filename of the image
	 */
	public function index($category, $name) {
		$info = array();
		$path = ensureslash($this->conf->get('image_dir') . $category) .
			$name;
		// get some data off the image
		$getimagesize = @getimagesize($path);
		$info['format'] = $getimagesize['mime'];
		$info['width'] = $getimagesize[0];
		$info['height'] = $getimagesize[1];
		$info['size'] = @filesize($path);

		// and some more from the EXIF headers
		$exif = @exif_read_data($path, 'IFD0');
		if($exif !== false && $info['format'] == 'image/jpeg') {
			$exif = exif_read_data($path, true, 0);
			if(isset($exif['DateTime'])) {
				$info['taken'] = $exif['DateTime'];
			}
			if(isset($exif['Make'])) {
				$info['cam_manufacturer'] = $exif['Make'];
			}
			if(isset($exif['Model'])) {
				$info['cam_model'] = $exif['Model'];
			}
			if(isset($exif['ShutterSpeedValue'])) {
				$info['shutter_speed'] =
					$exif['ShutterSpeedValue'];
			}
			if(isset($exif['SubjectDistance'])) {
				$info['subject_distance'] =
					$exif['SubjectDistance'];
			}
			if(isset($exif['FocalLength'])) {
				$info['focal_length'] = $exif['FocalLength'];
			}
			if(isset($exif['ExposureIndex'])) {
				$info['exposure_time'] = $exif['ExposureIndex'];
			}
		}
		try {
			$this->query->addImage($category, $name, $info);
			$this->thumbnail($category, $name);
		} catch(Exception $e) {
			throw $e;
		}
	}

	/**
	 * Generates a HTML page and displays it to user
	 */
	public function doPage() {
		echo $this->getPage();
	}

	/**
	 * Generates a HTML page
	 *
	 * @return generated HTML
	 */
	public function getPage() {
		// try to create View
		try {
			$view = new View($this->conf, $this->s);
		} catch(File_Not_Found_Exception $e) {
			$this->errors[] =
				$this->s->get('template_not_found_error') .
				$e->getMessage();
			if($this->conf->get('theme') !=	'default') {
				try {
					$view = new View($this->conf, $this->s,
						true);
				} catch(File_Not_Found_Exception $ee) {
					// even default theme not found
					// TODO: do something
				}
			} else {
				// again, default theme not found
				// TODO: do something
			}
		}
		// if, for whatever reason, the query hasn't been initialized,
		// only errors may be shown (others will spout more errors)
		if($this->query !== NULL && $view !== NULL) {
			// determine, what to show
			$display = $this->conf->get('prefix') . 'display';
			$login = $this->conf->get('prefix') . 'login';
			$about = $this->conf->get('prefix') . 'about';
			try {
				// check if logged in, and print stuff
				// accordingly
				if($this->_loggedIn) {
						$view->doSidebarLogin(
							View::LOGIN_LOGOUT);
				} else {
					if($this->conf->get('public_login')) {
						$view->doSidebarLogin(
							View::LOGIN_LOGIN);
					} else {
						$view->doSidebarLogin(
							View::LOGIN_HIDDEN);
					}
				}
				// categories are always there
				$view->doCategories(
					$this->query->getCategories());
				$view->doSearch(); // ditto

				// finally pick what to show in content area
				if(isset($_GET[$login])) {
					$view->doLogin($this->_loggedIn);
				} else if(isset($_GET[$about])) {
					$info = array();
					if($this->conf->get('public_info')) {
						// gather some info
						$tmp = apache_get_version();
						if($tmp !== false) {
							$info[] = array(
								'Apache', $tmp);
						}
						$info[] = array('PHP',
							phpversion());
						$info[] = array($this->query->
							getDriverName(),
							$this->query->
							getDriverVersion());
					}
					$view->doAbout($info);
				} else if(isset($_GET[$display])) {
					$this->doImage($view);
				} else {
					$this->doThumbnails($view);
				}
			} catch(PDOException $e) {
				$this->errors[] = $e->getMessage();
			}
		}
		// View initialization can actually fail, too. In that case
		// show just a simple error message
		if($view === NULL) {
			return $this->s->get('missing_template');
		} else {
			$view->doErrors($this->errors, $this->notices);
			return $view->doPage();
		}
	}

	/**
	 * Creates thumbnails for a single image
	 * 
	 * @param dir  directory (relative to image_dir) of the image
	 * @param file filename of the image
	 */
	protected function thumbnail($dir, $file) {
		$original = NULL;
		$path = ensureslash($this->conf->get('image_dir') . $dir) .
			$file;
		// get info about the file
		$info = @getimagesize($path);
		if($info === false) {
			// not an image
			throw new Image_Exception(implode(' ',
				error_get_last()));
		}

		// calculate sizes
		$w = $info[0];
		$h = $info[1];
		$tw = $this->conf->get('thumb_width');
		$th = $this->conf->get('thumb_height');
		$iw = $this->conf->get('image_width');
		$ih = $this->conf->get('image_height');

		// thumb and image ratios can be set separately--thus they need
		// to be calculated separately
		// thumbnail
		$wratio = $w / $tw;
		$hratio = $h / $th;
		if($wratio > $hratio) {
			$thumbw = $tw;
			$thumbh = $thumbw * $h / $w;
		} else {
			$thumbh = $th;
			$thumbw = $thumbh * $w / $h;
		}

		// larger image
		$wratio = $w / $iw;
		$hratio = $h / $ih;
		if($wratio > $hratio) {
			$imagew = $iw;
			$imageh = $imagew * $h / $w;
		} else {
			$imageh = $ih;
			$imagew = $imageh * $w / $h;
		}

		if(function_exists('symlink')) {
			$copyfunc = 'symlink';
		} else {
			$copyfunc = 'copy';
		}

		// make sure directory exists
		$thumbdir = ensureslash($this->conf->get('thumb_dir') . $dir);
		if(!is_dir($thumbdir)) {
			// TODO: make subdirs have same permissions as
			// conf->thumb_dir
			mkdir($thumbdir, 0777, true);
		}

		// create small thumbnail image
		$thumbpath = $thumbdir . $file . '_sm.jpg';
		if($w < $tw && $h < $th) {
			// if the original pic is small enough, we can just copy
			if(@$copyfunc(realpath($path), $thumbpath) === false) {
				throw new Image_Exception(implode(' ',
					error_get_last()));
			}
		} else {
			// otherwise do some resizing

			// read the file to GD resource
			list(, $type) = explode('/', $info['mime']);
			$func = "imagecreatefrom$type";
			$original = @$func($path);
			if($original === NULL) {
				throw new Image_Exception(implode(' ',
					error_get_last()));
			}

			$thumb = @imagecreatetruecolor($thumbw, $thumbh);
			if($thumb === NULL) {
				throw new Image_Exception(implode(' ',
					error_get_last()));
			}
			if(@imagecopyresampled($thumb, $original, 0, 0, 0, 0,
					$thumbw, $thumbh, $w, $h) === false) {
				throw new Image_Exception(implode(' ',
					error_get_last()));
			}
			if(@imagejpeg($thumb, $thumbpath) === false) {
				throw new Image_Exception(implode(' ',
					error_get_last()));
			}
			@imagedestroy($thumb);
		}

		// create larger thumbnail image
		$imagepath = $thumbdir . $file . '_nm.jpg';
		if($w < $iw && $h < $ih) {
			// small original, just copy
			if(@$copyfunc(realpath($path), $imagepath) === false) {
				throw new Image_Exception(implode(' ',
					error_get_last()));
			}
		} else {
			$image = @imagecreatetruecolor($imagew, $imageh);
			if($image === NULL) {
				throw new Image_Exception(implode(' ',
					error_get_last()));
			}
			if(@imagecopyresampled($image, $original, 0, 0, 0, 0,
					$imagew, $imageh, $w, $h) === false) {
				throw new Image_Exception(implode(' ',
					error_get_last()));
			}
			if(@imagejpeg($image, $imagepath) === false) {
				throw new Image_Exception(implode(' ',
					error_get_last()));
			}
			@imagedestroy($image);
		}
		if($original !== NULL) {
			@imagedestroy($original);
		}
	}

	/**
	 * Displays an image and its information on the page
	 *
	 * @param view reference to instance of View to use
	 */
	protected function doImage(View $view) {
		$catstr = $this->conf->get('prefix') . 'cat';
		$displaystr = $this->conf->get('prefix') . 'display';

		if(!isset($_GET[$displaystr]) || $this->query === NULL ||
				$view === NULL) {
			// can't do
			return;
		}
		$cat = isset($_GET[$catstr]) ? $_GET[$catstr] : '';

		// get info about image
		$imageinfo = $this->query->getImageByName($cat,
			$_GET[$displaystr]);
		if($imageinfo === false) {
			// if that fails it means no such image exists
			$this->errors[] = $this->s->get('no_image_error') .
				$_GET[$displaystr];
		} else {
			// it exists, pull also its id, comments, tags & neighbors from
			// db
			$imageinfo['id'] = $this->query->getImageIdByName($cat,
				$_GET[$displaystr]);
			$comments = $this->query->getComments($imageinfo['id']);
			$tags = $this->query->getTagNamesByImageId(
				$imageinfo['id']);
			$neighbors = $this->getImagesNeighbors($imageinfo);

			// and let View do its job
			$view->doImage($imageinfo, $comments, $tags, $neighbors[0],
				$neighbors[1], $this->_loggedIn);
		}
	}

	protected function getImagesNeighbors($image) {
		$searchstr = $this->conf->get('prefix') . 'search';
		$catstr = $this->conf->get('prefix') . 'cat';

		// Search or category list? We must know, because we need to use
		// different methods of Query depending on type.
		if(isset($_GET[$searchstr])) {
			// search
			$search = $_GET[$searchstr];
			try {
				$imgs = $this->query->getAllImageNamesByCriteria($search);
			} catch(Exception $e) {
				$this->errors[] = $e->getMessage();
			}
			$pos = 0;
			for(; $pos < count($imgs); ++$pos) {
				if($imgs[$pos][0] == NULL) {
					$imgs[$pos][0] = '';
				}
				if($imgs[$pos][1] == $image['filename'] &&
						$imgs[$pos][0] == $image['category']) {
					break;
				}
			}
			if($pos > 0) {
				$prev = $this->query->getImageByName(
					$imgs[$pos - 1][0], $imgs[$pos - 1][1]);
			} else {
				$prev = NULL;
			}
			if($pos < count($imgs) - 1) {
				$next = $this->query->getImageByName(
					$imgs[$pos + 1][0], $imgs[$pos + 1][1]);
			} else {
				$next = NULL;
			}
		} else {
			// category
			$cat = isset($_GET[$catstr]) ? $_GET[$catstr] : '';
			try {
				$imgs = $this->query->getAllImageNamesInCategory($cat);
			} catch(Exception $e) {
				$this->errors[] = $e->getMessage();
			}
			$pos = array_search($image['filename'], $imgs);
			if($pos > 0) {
				$prev['filename'] = $imgs[$pos - 1];
				$prev['category'] = $cat;
			} else {
				$prev = NULL;
			}
			if($pos < count($imgs) - 1) {
				$next['filename'] = $imgs[$pos + 1];
				$next['category'] = $cat;
			} else {
				$next = NULL;
			}
		}
		return array($prev, $next);
	}

	/**
	 * Displays bunch of thumbnails on the page
	 *
	 * @param view reference to instance of View to use
	 */
	protected function doThumbnails(View $view) {
		if($this->query === NULL || $view === NULL) {
			return;
		}

		$searchstr = $this->conf->get('prefix') . 'search';
		$catstr = $this->conf->get('prefix') . 'cat';
		$startstr = $this->conf->get('prefix') . 'start';
		$lenstr = $this->conf->get('prefix') . 'length';

		$start = isset($_GET[$startstr]) ? $_GET[$startstr] : 0;
		$len = isset($_GET[$lenstr]) ? $_GET[$lenstr] :
			$this->conf->get('per_page');

		$imgs = array();
		$max = 0;
		// Search or category list? We must know, because we need to use
		// different methods of Query depending on type.
		if(isset($_GET[$searchstr])) {
			// search
			$search = $_GET[$searchstr];
			try {
				$max = $this->query->getImageCountByCriteria($search);
				$imgs = $this->query->getImageNamesByCriteria(
					$search, $start, $len);
			} catch(Exception $e) {
				$this->errors[] = $e->getMessage();
			}
			$header = $this->s->get('search_header') . $search;
		} else {
			// category
			$cat = isset($_GET[$catstr]) ? $_GET[$catstr] : '';
			try {
				$max = $this->query->getImageCountInCategory(
					$cat);
				$imgs = $this->query->getImageNamesInCategory(
					$cat, $start, $len);
			} catch(Exception $e) {
				$this->errors[] = $e->getMessage();
			}
			$header = $this->s->get('category_header');
			$header .= $cat != '' ? $cat :
				$this->s->get('root_category');
		}
		// this again, however, is the same for every type
		$view->doThumbnails($header, $imgs, $start, $len, $max);
	}
}
?>
