<?php
/*
 * Copyright (c) 2008 Karri "Blaze" Kahelin
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
require_once('Not_Connected_Exception.php');
require_once('Query.php');

/**
 * Database frontend class for SQLite
 *
 * All methods override superclass methods, see superclass for documentation.
 */
class SqliteQuery extends Query {
	public function __construct($prefix) {
		parent::__construct($prefix);

		// trigger code courtesy of
		// http://www.rcs-comp.com/site/index.php/view/Utilities-SQLite_foreign_key_trigger_generator
		$this->s['create_tables'] =
			"CREATE TABLE %prefix%pictures (
			id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL ,
			filename TEXT NOT NULL ,
			category TEXT NOT NULL ,
			added TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
			taken TIMESTAMP NULL DEFAULT NULL ,
			width SMALLINT UNSIGNED NULL DEFAULT NULL ,
			height SMALLINT UNSIGNED NULL DEFAULT NULL ,
			cam_manufacturer VARCHAR( 255 ) NULL DEFAULT NULL ,
			cam_model VARCHAR( 255 ) NULL DEFAULT NULL ,
			shutter_speed VARCHAR( 255 ) NULL DEFAULT NULL ,
			subject_distance VARCHAR( 255 ) NULL DEFAULT NULL ,
			focal_length VARCHAR( 255 ) NULL DEFAULT NULL ,
			exposure_time VARCHAR( 255 ) NULL DEFAULT NULL ,
			location VARCHAR( 255 ) NULL DEFAULT NULL ,
			photographer VARCHAR( 255 ) NULL DEFAULT NULL ,
			description TEXT NULL DEFAULT NULL ,
			format VARCHAR( 64 ) NOT NULL ,
			size INT UNSIGNED NOT NULL
			);

			CREATE TABLE %prefix%comments (
			id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL ,
			pic_id INTEGER NOT NULL
				CONSTRAINT fk_com_pic_id
				REFERENCES %prefix%pictures(id)
				ON DELETE CASCADE ,
			comment TEXT NOT NULL ,
			username TEXT NOT NULL ,
			added TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP
			);

			CREATE TABLE %prefix%tags (
			id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL ,
			name VARCHAR( 255 ) NOT NULL
			);

			CREATE TABLE %prefix%tag_pic (
			tag_id INTEGER NOT NULL
				CONSTRAINT fk_tag_id
				REFERENCES %prefix%tags(id)
				ON DELETE CASCADE ,
			pic_id INTEGER NOT NULL
				CONSTRAINT fk_pic_id
				REFERENCES %prefix%pictures(id)
				ON DELETE CASCADE
			);";
		$this->s['create_triggers'] = "
		-- Foreign Key Preventing insert
		CREATE TRIGGER fki_%prefix%comments_pic_id_%prefix%pictures_id
		BEFORE INSERT ON [%prefix%comments]
		FOR EACH ROW BEGIN
		  SELECT RAISE(ROLLBACK, 'insert on table \"%prefix%comments\" violates foreign key constraint \"fki_%prefix%comments_pic_id_%prefix%pictures_id\"')
		  WHERE (SELECT id FROM %prefix%pictures WHERE id = NEW.pic_id) IS NULL;
		END;

		-- Foreign key preventing update
		CREATE TRIGGER fku_%prefix%comments_pic_id_%prefix%pictures_id
		BEFORE UPDATE ON [%prefix%comments] 
		FOR EACH ROW BEGIN
		    SELECT RAISE(ROLLBACK, 'update on table \"%prefix%comments\" violates foreign key constraint \"fku_%prefix%comments_pic_id_%prefix%pictures_id\"')
		      WHERE (SELECT id FROM %prefix%pictures WHERE id = NEW.pic_id) IS NULL;
		END;

		-- Cascading Delete
		CREATE TRIGGER fkdc_%prefix%comments_pic_id_%prefix%pictures_id
		BEFORE DELETE ON %prefix%pictures
		FOR EACH ROW BEGIN 
		    DELETE FROM %prefix%comments WHERE %prefix%comments.pic_id = OLD.id;
		END;

		-- Foreign Key Preventing insert
		CREATE TRIGGER fki_%prefix%tag_pic_tag_id_%prefix%tags_id
		BEFORE INSERT ON [%prefix%tag_pic]
		FOR EACH ROW BEGIN
		  SELECT RAISE(ROLLBACK, 'insert on table \"%prefix%tag_pic\" violates foreign key constraint \"fki_%prefix%tag_pic_tag_id_%prefix%tags_id\"')
		  WHERE (SELECT id FROM %prefix%tags WHERE id = NEW.tag_id) IS NULL;
		END;

		-- Foreign key preventing update
		CREATE TRIGGER fku_%prefix%tag_pic_tag_id_%prefix%tags_id
		BEFORE UPDATE ON [%prefix%tag_pic] 
		FOR EACH ROW BEGIN
		    SELECT RAISE(ROLLBACK, 'update on table \"%prefix%tag_pic\" violates foreign key constraint \"fku_%prefix%tag_pic_tag_id_%prefix%tags_id\"')
		      WHERE (SELECT id FROM %prefix%tags WHERE id = NEW.tag_id) IS NULL;
		END;

		-- Cascading Delete
		CREATE TRIGGER fkdc_%prefix%tag_pic_tag_id_%prefix%tags_id
		BEFORE DELETE ON %prefix%tags
		FOR EACH ROW BEGIN 
		    DELETE FROM %prefix%tag_pic WHERE %prefix%tag_pic.tag_id = OLD.id;
		END;

		-- Foreign Key Preventing insert
		CREATE TRIGGER fki_%prefix%tag_pic_pic_id_%prefix%pictures_id
		BEFORE INSERT ON [%prefix%tag_pic]
		FOR EACH ROW BEGIN
		  SELECT RAISE(ROLLBACK, 'insert on table \"%prefix%tag_pic\" violates foreign key constraint \"fki_%prefix%tag_pic_pic_id_%prefix%pictures_id\"')
		  WHERE (SELECT id FROM %prefix%pictures WHERE id = NEW.pic_id) IS NULL;
		END;

		-- Foreign key preventing update
		CREATE TRIGGER fku_%prefix%tag_pic_pic_id_%prefix%pictures_id
		BEFORE UPDATE ON [%prefix%tag_pic] 
		FOR EACH ROW BEGIN
		    SELECT RAISE(ROLLBACK, 'update on table \"%prefix%tag_pic\" violates foreign key constraint \"fku_%prefix%tag_pic_pic_id_%prefix%pictures_id\"')
		      WHERE (SELECT id FROM %prefix%pictures WHERE id = NEW.pic_id) IS NULL;
		END;

		-- Cascading Delete
		CREATE TRIGGER fkdc_%prefix%tag_pic_pic_id_%prefix%pictures_id
		BEFORE DELETE ON %prefix%pictures
		FOR EACH ROW BEGIN 
		    DELETE FROM %prefix%tag_pic WHERE %prefix%tag_pic.pic_id = OLD.id;
		END;";

	}

	public function createTables() {
		if(!$this->tablesExist()) {
			// seems to be the same as Postgres: can only create
			// one table per query, so we need to split:
			$queries = explode(';', $this->s['create_tables']);
			$queries[] = $this->s['create_triggers'];
			foreach($queries as $q) {
				if(strlen(trim($q)) == 0) {
					continue;
				}
				$stmt = $this->pdo->query(trim($q));
				if($stmt === false) {
					throw new PDOException(implode(' ',
						$this->pdo->errorInfo()));
				}
			}
		}
	}
}
