<?php
/*
 * Copyright (c) 2008 Karri "Blaze" Kahelin
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

require_once('Comment_Exception.php');
require_once('Admin_Exception.php');

/**
 * Handles form submits
 */
class Submit_Handler {
	protected $parent = NULL;
	protected $s = NULL;
	protected $db = NULL;
	protected $prefix = '';
	protected $givenPass = false;

	/**
	 * Constructs new Submit_Handler-object
	 *
	 * @param parent  instance of Sol_Gal to report (error) messages to
	 * @param strings instance of Strings to get the (error) messages from
	 * @param db      instance of Query to use
	 * @param prefix  prefix to use
	 */
	public function __construct(Sol_Gal $parent, Strings $strings,
			Query $db, $prefix) {
		$this->parent = $parent;
		$this->s = $strings;
		$this->db = $db;
		$this->prefix = $prefix;

		if(isset($_SESSION[$prefix . 'login'])) {
			$this->givenPass = $_SESSION[$prefix . 'login'];
		}
	}

	/**
	 * Handles form submits
	 */
	public function handleSubmits() {
		$prefix = $this->prefix;
		// find out if there's a form submitted for this prefix

		if(isset($_POST[$prefix . 'comment_delete'])) {
			// comment deletion

			$this->handleDeleteComment();
		} else if(isset($_POST[$prefix . 'comment_modify'])) {
			// comment edit

			$this->handleChangeComment();
		} else if(isset($_POST[$prefix . 'name'])) {
			// okay, a comment form
			$this->handleComment();

		} else if(isset($_POST[$prefix . 'new_tags'])) {
			// tags
			$this->handleTags();

		} else if(isset($_POST[$prefix . 'login'])) {
			// a-ha, a login form

			session_start();
			$this->givenPass = $_SESSION[$prefix . 'login'] =
				$_POST[$prefix . 'login'];
			$_GET[$prefix . 'session'] = 'yes';

		} else if(isset($_POST[$prefix . 'logout'])) {
			// logout button/form

			unset($_SESSION[$prefix . 'login']);
			$this->givenPass = false;

		} else if(isset($_POST[$prefix . 'imageinfo'])) {
			// image information form
			
			$this->handleImageInformation();
		}
	}

	/**
	 * Returns the password user has authenticated with
	 *
	 * @return password given by user
	 */
	public function getGivenPass() {
		return $this->givenPass;
	}

	/**
	 * Handles submission of comment form
	 *
	 * @return true if comment was successfully inserted
	 */
	protected function handleComment() {
		$prefix = $this->prefix;

		if(!isset($_GET[$prefix . 'display'])) {
			// no image specified
			throw new Comment_Exception('No display set');
		}
		$name = $_GET[$prefix . 'display'];
		$catstr = $prefix . 'cat';
		$category = isset($_GET[$catstr]) ?
			$_GET[$catstr] : '';

		try {
			// check each field contains something
			if(isset($_POST[$prefix . 'name']) &&
					strlen($_POST[$prefix .
					'name']) > 0 && isset($_POST[$prefix .
					'comment']) && strlen($_POST[$prefix .
					'comment']) > 0) {
				$user_com = $_POST[$prefix . 'name'];
				$comment_com = $_POST[$prefix . 'comment'];
				$pic_id = $this->db->getImageIdByName($category,
					$name);
				if($pic_id != 0) {
					$this->db->insertComment($user_com,
						$comment_com, $pic_id);
				} else {
					// no such image exists
					$this->parent->displayError(
						$this->s->get('no_image_error')
						. $category . $name);
				}
				$this->parent->displayNotice($this->s->get(
					'comment_saved'));
				return true;
			} else {
				$this->parent->displayError($this->s->get(
					'comment_error') .
					$this->s->get('empty_fields')
				);
				return false;
			}
		} catch(PDOException $e) {
			throw new Comment_Exception($e->getMessage);
		}
	}

	/**
	 * Handles deletion of a comment
	 */
	protected function handleDeleteComment() {
		$prefix = $this->prefix;

		$idstr = $prefix . 'comment_id';
		if(!isset($_POST[$idstr])) {
			throw new Admin_Exception('No id set');
		}
		$this->db->deleteComment($_POST[$idstr]);
		$this->parent->displayNotice($this->s->get('changes_saved'));
	}

	/**
	 * Handles changing a comment
	 *
	 * @return true if comment was successfully changed
	 */
	protected function handleChangeComment() {
		$prefix = $this->prefix;

		$idstr = $prefix . 'comment_id';
		$userstr = $prefix . 'name';
		$commentstr = $prefix . 'comment';
		// check there's an image
		if(isset($_POST[$idstr]) && $_POST[$idstr] != 0) {
			// and no empty fields
			if(isset($_POST[$userstr]) && $_POST[$userstr] != '' &&
					isset($_POST[$commentstr]) &&
					$_POST[$commentstr] != '') {
				$this->db->changeComment($_POST[$idstr],
					$_POST[$userstr], $_POST[$commentstr]);
				$this->parent->displayNotice(
					$this->s->get('changes_saved'));
				return true;
			} else {
				$this->parent->displayError($this->s->get(
					'comment_error') . $this->s->get(
					'empty fields'));
				return false;
			}
		} else {
			throw new Comment_Exception('id == 0');
		}
	}

	/**
	 * Handles changing tags of a picture
	 *
	 * @return true if tags were successfully changed
	 */
	protected function handleTags() {
		$prefix = $this->prefix;

		$displaystr = $prefix . 'display';
		$catstr = $prefix . 'cat';
		$tagstr = $prefix . 'new_tags';

		if(!isset($_GET[$displaystr])) {
			// no image was specified
			throw new Admin_Exception('No display set');
		}
		$category = isset($_GET[$catstr]) ? $_GET[$catstr] : '';
		$name = $_GET[$displaystr];
		try {
			$id = $this->db->getImageIdByName($category, $name);
			if($id != 0) {
				$tags = explode(',', $_POST[$tagstr]);
				for($i = 0; $i < count($tags); ++$i) {
					$tags[$i] = trim($tags[$i]);
				}
				$this->db->changeTags($id, $tags);
				$this->parent->displayNotice($this->s->get(
					'changes_saved'));
				return true;
			} else {
				// no such image
				$this->parent->displayError($this->s->get(
					'no_image_error') . $category . $name);
				return false;
			}
		} catch(PDOException $e) {
			throw new Admin_Exception($e->getMessage());
		}
	}

	/**
	 * Handles submission of image information form
	 *
	 * @return true if data was successfully changed
	 */
	protected function handleImageInformation() {
		$prefix = $this->prefix;

		$displaystr = $prefix . 'display';
		$catstr = $prefix . 'cat';

		if(!isset($_GET[$displaystr])) {
			// no image given
			throw new Admin_Exception('No display set');
		}
		$category = isset($_GET[$catstr]) ? $_GET[$catstr] : '';
		$name = $_GET[$displaystr];
		try {
			$id = $this->db->getImageIdByName($category, $name);
			if($id != 0) {
				$values = array();
				foreach($_POST as $key => $value) {
					if(strpos($key, $prefix) === 0 &&
							$key != $prefix .
							'imageinfo') {
						$values[substr($key,
							strlen($prefix))] =
							$value;
					}
				}
				$this->db->changeImage($id, $values);
				$this->parent->displayNotice($this->s->get(
					'changes_saved'));
				return true;
			} else {
				// no such image
				$this->parent->displayError($this->s->get(
					'no_image_error') . $category . $name);
				return false;
			}
		} catch(PDOException $e) {
			throw new Admin_Exception($e->getMessage());
		}
	}
}
?>
