<?php
/*
 * Copyright (c) 2008-2009 Karri "Blaze" Kahelin
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
define('SG_VERSION', '1.1.2');
define('SG_THEME_DIR', './themes/');

require_once('utils.php');
require_once('File_Not_Found_Exception.php');

/**
 * View class
 */
class View {
	const LOGIN_HIDDEN = 0;
	const LOGIN_LOGIN = 1;
	const LOGIN_LOGOUT = 2;

	protected $templates = array();
	protected $conf = NULL;
	protected $s = NULL;
	private $_errorsDone = false;
	private $_categoriesDone = false;
	private $_contentDone = false;
	private $_loginDone = false;
	private $_searchDone = false;

	/**
	 * Constructs new View-object
	 *
	 * @param settings instance of Settings to use
	 * @param string   instance of Strings to use
	 * @param default  override the theme setting found in settings (for
	 * 		   cases where incorrect theme is specified)
	 */
	public function __construct(Settings $settings, Strings $strings,
			$default = false) {
		$this->conf = $settings;
		$this->s = $strings;
		// load all templates
		$temps = array('page', 'error', 'notice', 'category', 'search',
			'thumbnails', 'thumbnail', 'image', 'imageinfos',
			'imageinfo', 'tags', 'tag', 'comment', 'comment.admin');
		$theme = $default ? 'default' : $this->conf->get('theme');
		foreach($temps as $template) {
			$filename = SG_THEME_DIR . $theme . '/' . $template .
				'.template';
			$this->templates[$template] =
				@file_get_contents($filename);
			if($this->templates[$template] === false) {
				if($template == 'page') {
					$this->templates['page'] =
						'%sg_errors%';
					throw new File_Not_Found_Exception(
						$filename);
				} else if($template == 'error') {
					$this->templates['error'] =
						'<div class="sg-error">' .
						'<strong>Error:</strong> ' .
						"%sg_error%</div>\n";
					throw new File_Not_Found_Exception(
						$filename);
				} else {
					$this->templates[$template] = '';
					throw new File_Not_Found_Exception(
						$filename);
				}
			}
		}
	}

	/**
	 * Displays errors on page
	 *
	 * @param errors  array of error messages to display
	 * @param notices array of informative messages to display
	*/
	public function doErrors(array $errors, array $notices) {
		$errString = '';
		if(count($errors) > 0) {
			foreach($errors as $err) {
				$errString .= str_replace('%sg_error%',
					htmlspecialchars($err),
					$this->templates['error']);
			}
		}
		if(count($notices) > 0) {
			foreach($notices as $note) {
				$errString .= str_replace('%sg_notice%',
					htmlspecialchars($note),
					$this->templates['notice']);
			}
		}
		$this->templates['page'] = str_replace('%sg_errors%',
			$errString, $this->templates['page']);
		$this->_errorsDone = true;
	}

	/**
	 * Displays category navigator on page
	 *
	 * @param cats 	array of categories to display
	 */
	public function doCategories(array $cats) {
		$str = '';
		$pstring = $this->stripQueryString();
		foreach($cats as $cat) {
			$tmp = str_replace('%sg_category_url%', $pstring .
				$this->conf->get('prefix') . 'cat=' .
				$cat, $this->templates['category']);
			if($cat == '') {
				$cat = $this->s->get('root_category');
			}
			$str .= str_replace('%sg_category%', $cat, $tmp);
		}
		$this->templates['page'] = str_replace('%sg_categories%', $str,
			$this->templates['page']);

		$this->_categoriesDone = true;
	}

	/**
	 * Displays thumbnails on page
	 *
	 * @param header 	text to display on page
	 * @param images 	array of image names to display
	 * @param start 	index of first thumbnail to display
	 * @param length 	amount of thumbnails per page
	 * @param max 		amount of thumbnail existing
	 * @param admin 	display administrator controls
	 */
	public function doThumbnails($header, array $images, $start, $length,
			$max, $admin = false) {
		$str = '';
		$pstring = $this->stripQueryString();
		$prefix = $this->conf->get('prefix');
		$searchstr = $prefix . 'search';
		$startstr = $prefix . 'start';
		$catstr = $prefix . 'cat';
		if(isset($_GET[$searchstr])) {
			$pstring .= $prefix . 'search=' .
				urlencode($_GET[$searchstr]) . '&amp;';
		}
		if(isset($_GET[$catstr])) {
			$pstring .= $prefix . 'cat=' .
				urlencode($_GET[$catstr]) . '&amp;';
		}
		if(isset($_GET[$startstr])) {
			$pstring .= $prefix . 'start=' .
				urlencode($_GET[$startstr]) . '&amp;';
		}

		foreach($images as $image) {
			$linkurl = $pstring;
			if($image[0] != '') {
				$cat = ensureslash($image[0]);
				$linkurl .= $this->conf->get('prefix') .
					'cat=' . $image[0] . '&amp;';
			} else {
				$cat = '';
			}
			$linkurl .= $this->conf->get('prefix') . 'display=' .
				$image[1];
			$tmp = str_replace('%sg_image_link%', $linkurl,
				$this->templates['thumbnail']);
			$tmp = str_replace('%sg_thumb%',
				$this->conf->get('thumb_dir') . $cat .
				$image[1] . '_sm.jpg', $tmp);
			$str .= str_replace('%sg_image_name%', $image[1], $tmp);
		}
		$string = str_replace('%sg_thumbnails%', $str,
			$this->templates['thumbnails']);
		$string = str_replace('%sg_header%', htmlspecialchars($header),
			$string);

		// navigation bar
		$str = '';
		// previous-link
		if($start > 0) {
			$newstart = $start - $length;
			if($newstart < 0) {
				$newstart = 0;
			}
			$str .= '<a href="' . $pstring .
				$this->conf->get('prefix') . 'start=' .
				$newstart . '">' . $this->s->get('prev') .
				'</a>' . $this->s->get('separator');
		} else {
			$str .= $this->s->get('prev') .
				$this->s->get('separator');
		}
		// page numbers
		for($st = 0, $pg = 1; $st < $max; $st += $length, ++$pg) {
			if($st - $start == 0) {
				// we're on this page--no link
				$str .= "$pg ";
			} else {
				$str .= '<a href="' . $pstring .
					$this->conf->get('prefix') . 'start=' .
					$st . '">' . $pg . '</a> ';
			}
		}
		$str .= $this->s->get('separator');
		// next-link
		if($start + $length < $max) {
			$newstart = $start + $length;
			$str .= '<a href="' . $pstring .
				$this->conf->get('prefix') . 'start=' .
				$newstart . '">' . $this->s->get('next') .
				'</a>';
		} else {
			$str .= $this->s->get('next');
		}
		$string = str_replace('%sg_navigator%', $str, $string);

		$this->templates['page'] = str_replace('%sg_content%', $string,
			$this->templates['page']);

		$replace = $this->s->get('title');
		if($category != '') {
			$replace .= " - $category";
		}
		$this->templates['page'] = str_replace('%sg_title%',
			htmlspecialchars($replace), $this->templates['page']);

		$this->_contentDone = true;
	}

	/**
	 * Displays log(in|out) form on page
	 *
	 * @param loggedId whether user is already logged in
	 */
	public function doLogin($loggedIn) {
		$str = '<h1>' . $this->s->get('login') . "</h1>\n";
		if($loggedIn) {
			$str .= '<form action="' . $this->getFormAction() .
				'" method="post"><input type="submit" name="' .
				$this->conf->get('prefix') . 'logout" value="' .
				$this->s->get('logout') . "\"/></form>\n";
		} else {
			$str .= '<form action="' . $this->getFormAction() .
				'" method="post"><input type="password" name="'
				. $this->conf->get('prefix') . 'login" ' .
				'/><input type="submit" value="' .
				$this->s->get('login') . "\" /></form>\n";
		}

		$this->templates['page'] = str_replace('%sg_content%', $str,
			$this->templates['page']);

		$replace = $this->s->get('title') . ' - ' .
			$this->s->get('login');
		$this->templates['page'] = str_replace('%sg_title%', $replace,
			$this->templates['page']);

		$this->_loginDone = true;
	}

	/**
	 * Displays search field on page
	 */
	public function doSearch() {
		$str = str_replace('%sg_form_search_search%',
			$this->conf->get('prefix') . 'search',
			$this->templates['search']);
		$str = str_replace('%sg_form_action%', $this->getFormAction(),
			$str);

		$this->templates['page'] = str_replace('%sg_search%', $str,
			$this->templates['page']);
		$this->_searchDone = true;
	}

	/**
	 * Displays information about the application on page
	 *
	 * @param info optional information displayed if public_info -setting
	 *             is true
	 */
	public function doAbout($info = array()) {
		$str = '<h1>' . $this->s->get('app_name') . ' ' . SG_VERSION .
			"</h1>\n";
		$str .= '<p>' . $this->s->get('copyright') . "</p>\n";
		$str .= $this->s->get('licence');
		if($this->conf->get('public_info')) {
			$tmp = '<table>';
			foreach($info as $in) {
				$tmp .= '<tr><td>' . htmlspecialchars($in[0]) .
					'</td><td>' . htmlspecialchars($in[1]) .
					"</tr>\n";
			}
			$tmp .= "</table>\n";
			$str .= $tmp;
		}
		$this->templates['page'] = str_replace('%sg_content%', $str,
			$this->templates['page']);

		$replace = $this->s->get('title') . ' - ' .
			$this->s->get('about');
		$this->templates['page'] = str_replace('%sg_title%', $replace,
			$this->templates['page']);

		$this->_contentDone = true;
	}


	/**
	 * Displays a single image and the info about it on page
	 *
	 * @param imageinfo associated array containing info about image
	 * @param comments  array of comments this image has
	 * @param tags      array of tags this image has
	 * @param prev      previous image in the set (NULL for last image in a set)
	 * @param next      next image in the set (NULL for first image in a set)
	 * @param admin     display administrator controls
	 */
	public function doImage(array $imageinfo, array $comments, array $tags,
			$prev, $next, $admin = false) {

		$pstring = $this->stripQueryString();
		$prefix = $this->conf->get('prefix');
		$catstr = $prefix . 'cat';
		$searchstr = $prefix . 'search';
		$startstr = $prefix . 'start';
		if(isset($_GET[$searchstr])) {
			$pstring .= $prefix . 'search=' .
				urlencode($_GET[$searchstr]) . '&amp;';
		} else if(isset($_GET[$catstr])) {
			$pstring .= $prefix . 'cat=' .
				urlencode($_GET[$catstr]) . '&amp;';
		}
		if(isset($_GET[$startstr])) {
			$pstring .= $prefix . 'start=' .
				urlencode($_GET[$startstr]) . '&amp;';
		}

		$str = str_replace('%sg_image_link%',
			ensureslash($this->conf->get('image_dir') .
			$imageinfo['category']) .
			$imageinfo['filename'],
			$this->templates['image']);
		$str = str_replace('%sg_image%',
			ensureslash($this->conf->get('thumb_dir') .
			$imageinfo['category']) . $imageinfo['filename'] .
			'_nm.jpg', $str);
		$str = str_replace('%sg_comments%',
			$this->doComments($comments, $admin), $str);
		$str = str_replace('%sg_form_comment_name%',
			$this->conf->get('prefix') . 'name', $str);
		$str = str_replace('%sg_form_comment_comment%',
			$this->conf->get('prefix') . 'comment', $str);
		$str = str_replace('%sg_form_comment_captcha%',
			$this->conf->get('prefix') . 'captcha', $str);
		$str = str_replace('%sg_imageinfos%', $this->doImageinfo(
			$imageinfo, $admin), $str);
		$str = str_replace('%sg_tags%', $this->doTags($tags, $admin),
			$str);
		$str = str_replace('%sg_image_name%', $imageinfo['filename'],
			$str);
		$str = str_replace('%sg_form_action%', $this->getFormAction(),
			$str);

		$this->templates['page'] = str_replace('%sg_content%', $str,
			$this->templates['page']);

		$this->templates['page'] = str_replace('%sg_title%',
			$this->s->get('title') . ' - ' . $imageinfo['filename'],
			$this->templates['page']);

		// navigation bar
		// back
		$str = '<a href="' . $pstring . '">' .
			$this->s->get('back') . '</a>' . $this->s->get('separator');
		// prev & next
		if($prev == NULL) {
			$str .= $this->s->get('prev') . $this->s->get('separator');
		} else {
			$str .= '<a href="' . $pstring . $this->conf->get('prefix') .
				'display=' . $prev['filename'];
			if(isset($_GET[$searchstr])) {
				$str .= '&amp;' . $prefix . 'cat=' . $prev['category'];
			}
			$str .= '">' . $this->s->get('prev') . '</a>' .
				$this->s->get('separator');
		}
		if($next == NULL) {
			$str .= $this->s->get('next');
		} else {
			$str .= '<a href="' . $pstring . $this->conf->get('prefix') .
				'display=' . $next['filename'];
			if(isset($_GET[$searchstr])) {
				$str .= '&amp;' . $prefix . 'cat=' . $next['category'];
			}
			$str .= '">' . $this->s->get('next') . '</a>';
		}

		$this->templates['page'] = str_replace('%sg_navigator%', $str,
			$this->templates['page']);

		$this->_contentDone = true;
	}

	/**
	 * Displays login form on the page
	 *
	 * @param mode type of form to display
	 */
	public function doSidebarLogin($mode = self::LOGIN_HIDDEN) {
		if($mode == self::LOGIN_LOGIN) {
			$repl = '<form action="' . $this->getFormAction() .
				'" method="post"><input type="password" name="'
				. $this->conf->get('prefix') . 'login" ' .
				'/><input type="submit" value="' .
				$this->s->get('login') . "\" /></form>\n";
		} else if($mode == self::LOGIN_LOGOUT) {
			$repl = '<form action="' . $this->getFormAction() .
				'" method="post"><input type="submit" name="' .
				$this->conf->get('prefix') . 'logout" value="' .
				$this->s->get('logout') . "\"/></form>\n";
		} else {
			$repl = '';
		}
		$this->templates['page'] = str_replace('%sg_login%', $repl,
			$this->templates['page']);

		$this->_loginDone = true;
	}

	/**
	 * Returns ready-to-print version of the page
	 *
	 * @return ready-to-print HTML
	 */
	public function doPage() {
		// first check everything is done, and if not, do them
		if(!$this->_errorsDone) {
			// assume no errors
			$this->doErrors(array(), array());
		}
		if(!$this->_loginDone) {
			$this->doLogin(false);
		}
		if(!$this->_searchDone) {
			$this->doSearch();
		}
		if(!$this->_categoriesDone) {
			$this->doCategories(array());
		}
		if(!$this->_contentDone) {
			// assume no content :P
			$this->templates['page'] = str_replace('%sg_content%',
				'', $this->templates['page']);
		}
		// TODO: replace with a nice graphical button
		$this->templates['page'] = str_replace('%sg_poweredby%',
			'Powered by <a href="http://blaze.kapsi.fi/solgal/">' .
			'SolGal/' . SG_VERSION . '</a>',
			$this->templates['page']);
		return $this->templates['page'];
	}

	/**
	 * Returns query string stripped of all params belonging to this
	 * instance of SolGal, EXCEPT %prefix%session
	 *
	 * @param appendable return string in appendable form (&amp; in the end)
	 *
	 * @return stripped query string
	 */
	protected function stripQueryString($appendable = true) {
		$parray = array();
		$prefix =  $this->conf->get('prefix');
		foreach($_GET as $key => $val) {
			if(strpos($key, $prefix) !== 0 && $key != $prefix .
					'session') {
				$parray[] = "$key=$val";
			}
		}
		$pstring = '?' . implode('&amp;', $parray);
		if($appendable && count($parray) > 0) {
			$pstring .= '&amp;';
		}
		return $pstring;
	}

	/**
	 * Displays information about the image on page
	 *
	 * @param imageinfo 	array containing image information
	 * @param admin 	display administrator controls
	 *
	 * @return string containing HTML for information about the image
	 */
	protected function doImageinfo(array $imageinfo, $admin = false) {
		// this is the order we try to display the data
		$order = array('taken', 'added', 'location', 'photographer',
			'cam_manufacturer', 'cam_model', 'shutter_speed',
			'subject_distance', 'focal_length', 'exposure_time',
			'width', 'height', 'format', 'size', 'description');
		if(get_magic_quotes_gpc()) {
			foreach($imageinfo as $key => $value) {
				$imageinfo[$key] = stripslashes($value);
			}
		}
		if($admin) {
			$prefix = $this->conf->get('prefix');
			$str = '<form action="' . $this->getFormAction() .
				'" method="post">';
			foreach($order as $key) {
				$tmp = str_replace('%sg_imageinfo_header%',
					'<label for="' . $prefix . $key . '">' .
					$this->s->get("ii_$key") . '</label>',
					$this->templates['imageinfo']);
				if($key == 'description') {
					$str .= str_replace(
						'%sg_imageinfo_data%',
						'<textarea rows="8" cols="21"' .
						' name="' . $prefix . $key .
						'" id="' . $prefix . $key . '">'
						. (isset($imageinfo[$key]) ?
						$imageinfo[$key] : '') .
						'</textarea>', $tmp);
				} else {
					$str .= str_replace(
						'%sg_imageinfo_data%',
						'<input name="' . $prefix .
						$key . '" id="' . $prefix .
						$key . '" value="' .
						(isset($imageinfo[$key]) ?
						$imageinfo[$key] : '') . '" />',
						$tmp);
				}
			}
			$str = '<form action="' . $this->getFormAction() .
				'" method="post">' .
				str_replace('%sg_imageinfo%', $str,
					$this->templates['imageinfos']);
			$str .= '<input type="submit" name="' . $prefix .
				'imageinfo" value="' . $this->s->get('apply') .
				"\" />\n</form>\n";

		} else {
			$str = '';
			foreach($order as $key) {
				if(isset($imageinfo[$key]) &&
						$imageinfo[$key] !== NULL &&
						$imageinfo[$key] !== '') {
					$tmp = str_replace(
						'%sg_imageinfo_header%',
						$this->s->get("ii_$key"),
						$this->templates['imageinfo']);
					// do some simple formatting
					if($key == 'size') {
						$val = ((int)($imageinfo[$key] /
							1000)) . 'kB';
					} else if($key == 'width' || $key ==
							'height') {
						$val = $imageinfo[$key] . 'px';
					} else if($key == 'taken' || $key ==
							'added') {
						$val = date('r', strtotime(
							$imageinfo[$key]));
					} else {
						$val = $imageinfo[$key];
					}
					$str .= str_replace(
						'%sg_imageinfo_data%', $val,
						$tmp);
				}
			}
			$str = str_replace('%sg_imageinfo%', $str,
				$this->templates['imageinfos']);
		}
		return $str;
	}

	/**
	 * Displays tags given for the image on page
	 *
	 * @param tags  array of tags
	 * @param admin display administrator controls
	 *
	 * @return string containing HTML for tags on image
	 */
	protected function doTags(array $tags, $admin = false) {
		if($admin) {
			$str = '<form action="' . $this->getFormAction() .
				'" method="post"><textarea name="' .
				$this->conf->get('prefix') . 'new_tags" rows=' .
				'"7" cols="40">';
			$str .= str_replace('%sg_tags%',
				$this->templates['tags'], implode(', ', $tags));
			$str .= "</textarea>\n<input type=\"submit\" value=\"" .
				$this->s->get('apply') . "\" />\n</form>\n";
			
		} else {
			$pstring = $this->stripQueryString();
			$str = '';
			foreach($tags as $tag) {
				$url = $pstring . $this->conf->get('prefix') .
					'search=' . $tag;
				$str .= str_replace('%sg_tag_link%',
					$url, str_replace('%sg_tag%', $tag,
					$this->templates['tag']));
			}
			$str = str_replace('%sg_tags%', $str,
				$this->templates['tags']);
		}
		return $str;
	}

	/**
	 * Displays comments given for the image on page
	 *
	 * @param comments array of comments
	 * @param admin    display administrator controls
	 *
	 * @return string containing HTML for comments on image
	 */
	protected function doComments(array $comments, $admin = false) {
		$str = '';
		if(get_magic_quotes_gpc()) {
			$comments = stripslashes_deep($comments);
		}
		if($admin) {
			foreach($comments as $comment) {
				$tmp = str_replace('%sg_user%',
					htmlspecialchars($comment['username']),
					$this->templates['comment.admin']);
				$tmp = str_replace('%sg_message%',
					htmlspecialchars($comment['comment']),
					$tmp);
				$tmp = str_replace('%sg_form_comment_id%',
					$this->conf->get('prefix') .
					'comment_id', $tmp);
				$tmp = str_replace('%sg_comment_id%',
					$comment['id'], $tmp);
				$tmp = str_replace('%sg_form_comment_delete%',
					$this->conf->get('prefix') .
					'comment_delete', $tmp);
				$tmp = str_replace('%sg_form_comment_modify%',
					$this->conf->get('prefix') .
					'comment_modify', $tmp);
				$str .= str_replace('%sg_added%',
					htmlspecialchars($comment['added']),
					$tmp);
			}
		} else {
			foreach($comments as $comment) {
				$tmp = str_replace('%sg_user%',	
					htmlspecialchars($comment['username']), 
					$this->templates['comment']);
				$tmp = str_replace('%sg_message%', 
					htmlspecialchars($comment['comment']),
					$tmp);
				$str .= str_replace('%sg_added%', 
					htmlspecialchars($comment['added']),
					$tmp);	
			}
		}
		return $str;
	}

	/**
	 * Returns valid URL for submitting a form
	 *
	 * @return url for submitting a form
	 */
	protected function getFormAction() {
		$sessionstr = $this->conf->get('prefix') . 'session';
		$uri = htmlspecialchars($_SERVER['REQUEST_URI']);
		if(isset($_GET[$sessionstr]) && $_GET[$sessionstr] == 'yes') {
			if(strpos($uri, $sessionstr . '=yes') === false) {
				$uri .= '&amp;' . $sessionstr . '=yes';
			}
		}
		return $uri;
	}
}
?>
